/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 *
 * Contributor(s):
 *
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package test.ejb.mdb;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.MessageDriven;
import javax.ejb.NoSuchEJBException;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.enterprise.inject.spi.BeanManager;
import javax.inject.Inject;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

import test.beans.TestApplicationScopedBean;
import test.beans.TestRequestScopedBean;
import test.ejb.session.Hello1;
import test.ejb.session.Hello2;

@TransactionManagement(TransactionManagementType.BEAN)
@MessageDriven(mappedName = "jms/cdi_hello_mdb_InQueue", description = "mymessagedriven bean description")
public class MessageBean implements MessageListener {

    @EJB
    private Hello1 hello1;
    @EJB
    private Hello2 hello2;

    @Resource(name = "jms/MyQueueConnectionFactory", mappedName = "jms/cdi_hello_mdb_QCF")
    QueueConnectionFactory qcFactory;

    @Resource(mappedName = "jms/cdi_hello_mdb_OutQueue")
    Queue clientQueue;

    @Inject
    TestApplicationScopedBean tasb;
    @Inject
    TestRequestScopedBean trsb;
    
    @Inject BeanManager bm;

    public void onMessage(Message message) {

        QueueConnection connection = null;
        try {

            System.out.println("Calling hello1 stateless bean");
            hello1.hello("local ejb3.0 stateless");
            System.out.println("Calling hello2 stateful bean");
            hello2.hello("local ejb3.0 stateful");
            hello2.removeMethod();
            try {
                hello2.hello("this call should not go through");
                throw new Exception("bean should have been removed "
                        + "after removeMethod()");
            } catch (NoSuchEJBException e) {
                System.out.println("Successfully caught EJBException after "
                        + " accessing removed SFSB");
            }

            System.out.println("Application scoped test bean:"+ tasb);
            System.out.println("Request scoped test bean:"+ trsb);
            if (tasb == null)
                throw new Exception(
                        "Injection of application scoped Bean in MDB failed");
            if (trsb == null)
                throw new Exception(
                        "Injection of request scoped Bean in MDB failed");
            
            connection = qcFactory.createQueueConnection();
            QueueSession session = connection.createQueueSession(false,
                    Session.AUTO_ACKNOWLEDGE);
            QueueSender sender = session.createSender(clientQueue);
            TextMessage tmessage = session.createTextMessage();
            tmessage.setText("mdb() invoked");
            System.out.println("Sending message");
            sender.send(tmessage);
            System.out.println("message sent");

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }

}
